@@ -91,12 +91,18 @@ class Composite(DeepEqual):
         """ Returns the number of key - value mappings in this map. """
         return len(self._sets)
 
+    def __len__(self):
+        """ Returns the number of key - value mappings in this map. """
+        return len(self._sets)
+
     def __repr__(self):
-        return self.__class__.__name__ + \
-            self._sets.__repr__()
+        return self.__class__.__name__ + '(' + (self._sets.__repr__() if hasattr(self, '_sets') else 'None') + ')'
 
     def toString(self, level=0, matprint=None, trans=True, **kwds):
-        return self.__class__.__name__ + self._sets.toString(level=level, matprint=matprint, trans=trans, **kwds)
+        return self.__class__.__name__ + \
+            self._sets.toString(level=level,
+                                tablefmt=tablefmt, tablefmt1=tablefmt1, tablefmt2=tablefmt2,
+                                matprint=matprint, trans=trans, **kwds)
 
     def __contains__(self, x):
         """ mh: enable 'x in composite' """
