@@ -5,6 +5,7 @@ import codecs
 import binascii
 import array
 import mmap
+from collections import ChainMap
 import builtins
 from collections import UserDict
 from collections.abc import MutableMapping as MM, MutableSequence as MS, MutableSet as MS
@@ -31,10 +32,8 @@ classes are defined to avoid circular dependency (such as ,
 Serializable.
 '''
 
-BD = builtins.__dict__
 
-
-def constructSerializable(obj, lgb=None, debug=False):
+def constructSerializable(obj, lookup=None, debug=False):
     """ mh: reconstruct object from the output of jason.loads().
     Recursively goes into nested class instances that are not
     encoded by default by JSONEncoder, instantiate and fill in
@@ -71,7 +70,7 @@ def constructSerializable(obj, lgb=None, debug=False):
                 print(spaces + 'looping through list %d <%s>' %
                       (i, xc.__name__))
             if issubclass(xc, (list, dict)):
-                des = constructSerializable(x, lgb=lgb, debug=debug)
+                des = constructSerializable(x, lookup=lookup, debug=debug)
             else:
                 des = x
             inst.append(des)
@@ -104,9 +103,9 @@ def constructSerializable(obj, lgb=None, debug=False):
                     print(spaces + 'Instanciate array.array')
                 indent -= 1
                 return inst
-        if classname in lgb:
+        if classname in lookup:
             # Now we have a blank instance.
-            inst = lgb[classname]()
+            inst = lookup[classname]()
             if debug:
                 print(spaces + 'Instanciate custom obj <%s>' % classname)
         elif classname == 'ellipsis':
@@ -114,11 +113,17 @@ def constructSerializable(obj, lgb=None, debug=False):
                 print(spaces + 'Instanciate Ellipsis')
             indent -= 1
             return Ellipsis
-        elif classname in BD and 'obj' in obj:
-            o = constructSerializable(obj['obj'], lgb=lgb, debug=debug)
-            inst = BD[classname](o)
+        elif classname in lookup and 'obj' in obj:
+            o = constructSerializable(obj['obj'], lookup=lookup, debug=debug)
+            inst = lookup[classname](o)
             if debug:
-                print(spaces + 'Instanciate builtin %s' % obj['obj'])
+                print(spaces + 'Instanciate defined %s' % obj['obj'])
+            indent -= 1
+            return inst
+        elif classname == 'dtype':
+            if debug:
+                print(spaces + 'Instanciate type %s' % obj['obj'])
+            inst = lookup[obj['obj']]
             indent -= 1
             return inst
         else:
@@ -136,7 +141,7 @@ def constructSerializable(obj, lgb=None, debug=False):
                 print(spaces + '[%s]value(dict/list) <%s>: %s' %
                       (k, v.__class__.__qualname__,
                        lls(list(iter(v)), 70)))
-            desv = constructSerializable(v, lgb=lgb, debug=debug)
+            desv = constructSerializable(v, lookup=lookup, debug=debug)
         else:
             if debug:
                 print(spaces + '[%s]value(simple) <%s>: %s' %
@@ -207,14 +212,14 @@ class IntDecoderOD(IntDecoder):
             return o
 
 
-def deserialize(js, lgb=None, debug=False, usedict=True):
+def deserialize(js, lookup=None, debug=False, usedict=True):
     """ Loads classes with _STID from the results of serialize.
 
     if usedict is True dict insted of ODict will be used.
     """
 
-    if lgb is None:
-        lgb = Classes.mapping
+    if lookup is None:
+        lookup = ChainMap(Classes.mapping, globals(), vars(builtins))
 
     if not isinstance(js, strset) or len(js) == 0:
         return None
@@ -236,4 +241,4 @@ def deserialize(js, lgb=None, debug=False, usedict=True):
 
     global indent
     indent = -1
-    return constructSerializable(obj, lgb=lgb, debug=debug)
+    return constructSerializable(obj, lookup=lookup, debug=debug)
