@@ -5,8 +5,10 @@ from ..utils.common import lls, ld2tk
 from .serializable import Serializable
 
 import pprint
+import array
 from functools import lru_cache
 from collections import OrderedDict
+from itertools import chain
 
 import logging
 # create logger
@@ -173,41 +175,33 @@ def deepcmp(obj1, obj2, seenlist=None, verbose=False, eqcmp=False):
     return run(obj1, obj2, v=verbose, eqcmp=eqcmp)
 
 
-class StateEqual(object):
-    """ Equality tested by hashed state.
+def xhash(hash_list=None):
+    """ get the hash of a tuple of hashes of all members of given sequence.
+
+    hash_list: use instead of self.getstate__()
     """
 
-    def __hash__(self):
+    hashes = []
+    if issubclass(hash_list.__class__, (str, bytes)):
+        # put str first so it is not treated as a sequence
+        return(hash(hash_list))
+    elif hasattr(hash_list, 'items'):
+        source = chain.from_iterable(hash_list.items())
+    elif hasattr(hash_list, '__iter__'):
+        source = hash_list
+    elif issubclass(hash_list.__class__, (array.array)):
+        source = (hash_list.typecode,
+                  hash_list.itemsize,
+                  len(hash_list),
+                  len(hash_list[0]))
+    else:
+        return(hash(hash_list))
 
-        @lru_cache(maxsize=16)
-        def cached_hash(t):
-            return hash(t)
-
-        t = tuple(self.__getstate__().values())
-        try:
-            return hash(t)
-        except TypeError:
-            """ lists/dicts recursively changed to tuples/frozensets before hashed """
-            t = ld2tk(t)
-            return hash(t)
-
-    def equals(self, obj, verbose=False, **kwds):
-        return self.__eq__(obj, verbose=verbose)
-
-    def __eq__(self, obj, verbose=False, **kwds):
-        """ compares hash. """
-        try:
-            h1, h2 = self.__hash__(), obj.__hash__()
-        except AttributeError:
-            return False
-        except TypeError:
-            return False
-        if verbose:
-            print('hashes ', h1, h2)
-        return h1 == h2
-
-    def a__ne__(self, obj):
-        return not self.__eq__(obj)
+    for t in source:
+        h = t.hash() if hasattr(t, 'hash') else xhash(t)
+        hashes.append(h)
+    # if there is only one element only hash the element
+    return hash(hashes[0] if len(hashes) == 1 else tuple(hashes))
 
 
 class DeepcmpEqual(object):
@@ -293,4 +287,43 @@ class EqualODict(object):
         return not self.__eq__(obj)
 
 
+class StateEqual(object):
+    """ Equality tested by hashed state.
+    """
+
+    def hash(self):
+        return xhash(self.__getstate__())
+
+    def __eq__(self, obj, verbose=False, **kwds):
+        """ compares hash. """
+
+        if obj is None:
+            return False
+
+        if id(self) == id(obj):
+            return True
+
+        if type(self) != type(obj):
+            return False
+        try:
+            h1, h2 = self.hash(), obj.hash()
+        except AttributeError:
+            return False
+        except TypeError:
+            return False
+        if verbose:
+            print('hashes ', h1, h2)
+        return h1 == h2
+
+    equals = __eq__
+
+    def __xne__(self, obj):
+        return not self.__eq__(obj)
+
+    def __hash__(self):
+        return self.hash()
+
+    __hash__ = hash
+
+
 DeepEqual = StateEqual
