@@ -2,13 +2,14 @@
 
 from ..utils.masked import masked
 from ..utils.common import grouper
+from ..utils.ydump import ydump
 from .serializable import Serializable
 from .datatypes import DataTypes, DataTypeNames, Vector, Vector2D, Quaternion
 from .odict import ODict
 from .composite import Composite
 from .listener import DatasetEventSender, ParameterListener, DatasetListener, DatasetEvent, EventTypeOf
 from .quantifiable import Quantifiable
-from .eq import DeepEqual
+from .eq import DeepEqual, xhash
 from .copyable import Copyable
 from .annotatable import Annotatable
 from .finetime import FineTime, FineTime1, utcobj
@@ -20,7 +21,8 @@ from ..utils.common import exprstrs, wls, mstr, t2l
 from tabulate import tabulate
 
 import builtins
-from collections import OrderedDict, Sequence
+from collections import OrderedDict
+from collections.abc import Sequence
 from numbers import Number
 import logging
 # create logger
@@ -208,10 +210,16 @@ class AbstractParameter(Annotatable, Copyable, DeepEqual, DatasetEventSender, Se
         """ Value as string for building the string representation of the parameter.  """
         return
 
-    def __repr__(self):
-        return self.toString(level=1)
+    def hash(self):
+        """ hash and equality derived only from the value of the parameter.
 
-    def toString(self, level=0, alist=False, **kwds):
+        because Python does not allow overriding __eq__ without setting hash to None.
+        """
+        return xhash(hash_list=self._value)
+
+    def toString(self, level=0,
+                 tablefmt='rst', tablefmt1='simple', tablefmt2='simple',
+                 alist=False, **kwds):
         """ alist: returns a dictionary string representation of parameter attributes.
         """
         vs = str(self._value)
@@ -230,7 +238,10 @@ class AbstractParameter(Annotatable, Copyable, DeepEqual, DatasetEventSender, Se
                            _STID=self._STID
                            )
 
-    __hash__ = DeepEqual.__hash__
+
+def make_jsonable(valid):
+
+    return[t2l([k, v]) for k, v in valid.items()] if issubclass(valid.__class__, dict) else t2l(valid)
 
 
 class Parameter(AbstractParameter, Typed):
@@ -369,11 +380,11 @@ f        With two positional arguments: arg1-> value, arg2-> description. Parame
     def setValid(self, valid):
         """ Sets the valid of this object.
 
-        If valid is None or empty, set as None, else save in a way so the tuple keys can be serialized with JSON.
+        If valid is None or empty, set as None, else save in a way so the tuple keys can be serialized with JSON. [[[rangelow, ranehi], state1], [[range2low, r..]..]..]
         """
 
         self._valid = None if valid is None or len(
-            valid) == 0 else [t2l([k, v]) for k, v in valid.items()] if issubclass(valid.__class__, dict) else t2l(valid)
+            valid) == 0 else make_jsonable(valid)
 
     def isValid(self):
         res = self.validate(self.value)
@@ -496,10 +507,9 @@ f        With two positional arguments: arg1-> value, arg2-> description. Parame
             return
         self._value = self.checked(value)
 
-    def __repr__(self):
-        return self.toString(level=1)
-
-    def toString(self, level=0, alist=False, **kwds):
+    def toString(self, level=0,
+                 tablefmt='rst', tablefmt1='simple', tablefmt2='simple',
+                 alist=False, **kwds):
 
         ret = exprstrs(self, level=level, **kwds)
         if alist:
@@ -510,6 +520,8 @@ f        With two positional arguments: arg1-> value, arg2-> description. Parame
         return '%s{ %s <%s>, "%s", default= %s, valid= %s tcode=%s}' % \
             (self.__class__.__name__, vs, ts, ds, fs, gs, cs)
 
+    __str__ = toString
+
     def __getstate__(self):
         """ Can be encoded with serializableEncoder """
         return OrderedDict(description=self.description,
@@ -521,8 +533,6 @@ f        With two positional arguments: arg1-> value, arg2-> description. Parame
                            _STID=self._STID
                            )
 
-    __hash__ = DeepEqual.__hash__
-
 
 class NumericParameter(Parameter, Quantifiable):
     """ has a number as the value, a unit, and a typecode.
@@ -532,9 +542,6 @@ class NumericParameter(Parameter, Quantifiable):
         super(NumericParameter, self).__init__(
             value=value, description=description, typ_=typ_, default=default, valid=valid, **kwds)
 
-    def __repr__(self):
-        return self.toString(level=1)
-
     def __getstate__(self):
         """ Can be encoded with serializableEncoder """
         return OrderedDict(description=self.description,
@@ -578,8 +585,6 @@ class NumericParameter(Parameter, Quantifiable):
                     'Sequence of only 2 to 4 elements for NumericParameter')
         super().setDefault(default)
 
-    __hash__ = DeepEqual.__hash__
-
 
 class DateParameter(Parameter):
     """ has a FineTime as the value.
@@ -628,10 +633,6 @@ class DateParameter(Parameter):
             default = FineTime(date=default, format=self.getTypecode())
         super().setDefault(default)
 
-    def __repr__(self):
-
-        return self.toString(level=1)
-
     def __getstate__(self):
         """ Can be encoded with serializableEncoder """
         return OrderedDict(description=self.description,
@@ -640,15 +641,10 @@ class DateParameter(Parameter):
                            valid=self._valid,
                            typecode=self.typecode,
                            _STID=self._STID)
-        return self.__class__.__name__ + \
-            '{ description = "%s", value = "%s", typecode = "%s"}' % \
-            (str(self.description), str(self.value), str(self.getTypecode()))
-
-    __hash__ = DeepEqual.__hash__
 
 
 class DateParameter1(DateParameter):
-    """ Like DateParameter but usese  FineTime. """
+    """ Like DateParameter but usese  FineTime1. """
 
     def setValue(self, value):
         """ accept any type that a FineTime1 does.
@@ -664,8 +660,6 @@ class DateParameter1(DateParameter):
             default = FineTime1(date=default, format=self.getTypecode())
         super().setDefault(default)
 
-    __hash__ = DeepEqual.__hash__
-
 
 class StringParameter(Parameter):
     """ has a unicode string as the value, a typecode for length and char.
@@ -692,9 +686,6 @@ class StringParameter(Parameter):
         """ Sets the typecode of this object. """
         self._typecode = typecode
 
-    def __repr__(self):
-        return self.toString(level=1)
-
     def __getstate__(self):
         """ Can be encoded with serializableEncoder """
         return OrderedDict(value=self._value,
@@ -704,8 +695,6 @@ class StringParameter(Parameter):
                            typecode=self._typecode,
                            _STID=self._STID)
 
-    __hash__ = DeepEqual.__hash__
-
 
 MetaHeaders = ['name', 'value', 'unit', 'type', 'valid',
                'default', 'code', 'description']
@@ -757,6 +746,10 @@ class MetaData(Composite, Copyable, Serializable, ParameterListener, DatasetEven
                              change=ch, cause=ca, rootCause=ro)
             self.fire(e)
 
+    def __repr__(self):
+
+        return ydump(self.__getstate__(), default_flow_style=True)
+
     def remove(self, name):
         """ add eventhandling """
         r = super(MetaData, self).remove(name)
@@ -774,11 +767,12 @@ class MetaData(Composite, Copyable, Serializable, ParameterListener, DatasetEven
             self.fire(e)
         return r
 
-    def toString(self, level=0, tablefmt='rst', tablefmt1='simple',
+    def toString(self, level=0,
+                 tablefmt='rst', tablefmt1='simple', tablefmt2='simple',
                  widths=None, **kwds):
         """ return  string representation of metada.
 
-        level: 0 is the most detailed, 2 is the least, 1 is for __repr__.
+        level: 0 is the most detailed, 2 is the least,
         tablefmt: format string in packae ``tabulate``.
         widths: controls how the attributes of every parameter are displayed in the table cells. If is set to -1, there is no cell-width limit. For finer control set a dictionary of parameter attitute names and how many characters wide its tsble cell is, 0 for ommiting the attributable. Default is
 ``{'name': 8, 'value': 17, 'unit': 7, 'type': 8,
@@ -806,7 +800,7 @@ class MetaData(Composite, Copyable, Serializable, ParameterListener, DatasetEven
                 tab.append(l)
             elif level == 1:
                 ps = '%s= %s' % (att['name'], att['value'])
-                # s += mstr(self, level=level, depth=1,  tablefmt = tablefmt, **kwds)
+                # s += mstr(self, level=level, tablefmt = tablefmt,                   tablefmt=tablefmt, tablefmt1=tablefmt1, tablefmt2=tablefmt2,depth=1, **kwds)
                 tab.append(wls(ps, 80//N))
                 if 0:
                     row.append(wls(ps, 80//N))
@@ -817,7 +811,10 @@ class MetaData(Composite, Copyable, Serializable, ParameterListener, DatasetEven
             else:
                 tab.append(att['name'])
 
-        lsnr = self.listeners.toString(level=level, **kwds)
+        lsnr = self.listeners.toString(level=level,
+                                       tablefmt=tablefmt, tablefmt1=tablefmt1,
+                                       tablefmt2=tablefmt2,
+                                       **kwds)
 
         # write out the table
         if level == 0:
@@ -836,10 +833,7 @@ class MetaData(Composite, Copyable, Serializable, ParameterListener, DatasetEven
             s += ', '.join(tab)
             return '%s, listeners = %s' % (s, lsnr)
         return '\n%s\n%s-listeners = %s' % (s, cn, lsnr) if len(tab) else \
-            '%s%s-listeners = %s' % ('(No parameter.)', cn, lsnr)
-
-    def __repr__(self, **kwds):
-        return self.toString(level=1, **kwds)
+            '%s %s-listeners = %s' % ('(No Parameter.)', cn, lsnr)
 
     def __getstate__(self):
         """ Can be encoded with serializableEncoder """
@@ -849,4 +843,3 @@ class MetaData(Composite, Copyable, Serializable, ParameterListener, DatasetEven
         return OrderedDict(_sets=self._sets,
                            listeners=self.listeners,
                            _STID=self._STID)
-    __hash__ = DeepEqual.__hash__
