@@ -2,13 +2,12 @@
 from collections import OrderedDict, UserDict
 from collections.abc import Collection
 from .serializable import Serializable
-from .eq import DeepEqual
+from .eq import DeepEqual, xhash
 from ..utils.common import bstr
 from ..utils.ydump import ydump
 
 from pprint import pformat
 import logging
-import pdb
 
 logger = logging.getLogger(__name__)
 
@@ -17,7 +16,7 @@ OD_toString_Nest = 0
 
 
 class ODict(UserDict, Serializable, DeepEqual):
-    """ Ordered dict that is not a subclass of dict and with a better __repr__.
+    """ Ordered dict that is not a subclass of dict and with a better __str__.
     """
 
     def __init__(self, *args, **kwds):
@@ -66,7 +65,9 @@ class ODict(UserDict, Serializable, DeepEqual):
     #             kk = tuple(item[0])
     #             self[kk] = item[1]
 
-    def toString(self, level=0, matprint=None, trans=True, **kwds):
+    def toString(self, level=0,
+                 tablefmt='rst', tablefmt1='simple', tablefmt2='simple',
+                 matprint=None, trans=True, **kwds):
         global OD_toString_Nest
 
         # return 'OD' + str(type(self.data))+'*'+str(self.data)
@@ -76,18 +77,21 @@ class ODict(UserDict, Serializable, DeepEqual):
         OD_toString_Nest += 1
         d = '<ODict '
         for n, v in self.data.items():
-            d += '    ' * OD_toString_Nest + '[ ' + str(n) + ' ]= '
-            s = bstr(v, level=level, matprint=matprint, trans=trans, **kwds)
-            d += s
+            #d += '    ' * OD_toString_Nest + '[ ' + str(n) + ' ]= '
+            d += '"' + str(n) + '":' + '\n' if level == 0 else ' '
+            s = bstr(v, level=level,
+                     tablefmt=tablefmt, tablefmt1=tablefmt1, tablefmt2=tablefmt2,
+                     matprint=matprint, trans=trans, **kwds)
+            d = d + s
         OD_toString_Nest -= 1
         return d + '>'
 
-    def __repr__(self):
-        """ returns string representation with details set according to debuglevel.
-        """
-        # return 'OD'+super().__repr__()
-        level = int(logger.getEffectiveLevel()/10) - 1
-        return self.toString(level=level)
+    # def __repr__(self):
+    #     """ returns string representation with details set according to debuglevel.
+    #     """
+    #     # return 'OD'+super().__repr__()
+    #     level = int(logger.getEffectiveLevel()/10) - 1
+    #     return self.toString(level=level)
 
     def __getstate__(self):
         """ Can be encoded with serializableEncoder """
@@ -104,6 +108,6 @@ class ODict(UserDict, Serializable, DeepEqual):
             _STID=self._STID
         )
 
-    def __hash__(self):
+    def hash(self):
 
-        return hash(tuple(self.data.items()))
+        return xhash(hash_list=self.data.items())
