@@ -21,13 +21,20 @@ class TM(MapContext):
 # sub-classing testing class
 # 'version' of subclass is int, not string
 
+sp = copy.deepcopy(PPI)
+sp['name'] = 'SP'
+sp['metadata']['version']['data_type'] = 'integer'
+sp['metadata']['version']['default'] = 9
+sp['metadata']['type']['default'] = sp['name']
+MdpInfo = sp['metadata']
+
 
 class SP(Product):
     def __init__(self,
                  description='UNKNOWN',
                  typ_='SP',
                  creator='UNKNOWN',
-                 version=9,
+                 version='9',
                  creationDate=FineTime(0),
                  rootCause='UNKNOWN',
                  startDate=FineTime(0),
@@ -35,24 +42,13 @@ class SP(Product):
                  instrument='UNKNOWN',
                  modelName='UNKNOWN',
                  mission='_AGS',
+                 zInfo=None,
                  **kwds):
         metasToBeInstalled = copy.copy(locals())
-        for x in ('self', '__class__', 'kwds'):
+        for x in ('self', '__class__', 'zInfo', 'kwds'):
             metasToBeInstalled.pop(x)
 
-        sp = {}
-        sp = copy.deepcopy(PPI)
-        sp['name'] = self.__class__.__name__
-        sp['metadata']['version']['data_type'] = 'integer'
-        sp['metadata']['version']['default'] = 9
-        sp['metadata']['type']['default'] = sp['name']
         self.zInfo = sp
         assert PPI['metadata']['version']['data_type'] == 'string'
-        super().__init__(metasToBeInstalled=metasToBeInstalled, **kwds)
-        super().installMetas(mtbi=metasToBeInstalled, prodInfo=self.zInfo)
-
-    @ property
-    def version(self): pass
-
-    @ version.setter
-    def version(self, p): pass
+        super().__init__(zInfo=zInfo, **metasToBeInstalled, **kwds)
+        # super().installMetas(metasToBeInstalled)
