@@ -8,8 +8,8 @@ from requests.auth import HTTPBasicAuth
 from fdi.dataset.serializable import serialize
 from fdi.dataset.deserialize import deserialize
 from fdi.pal.urn import parseUrn, parse_poolurl
-from .pnsconfig import pnsconfig as pcc
 from fdi.utils.getconfig import getConfig
+from ..pal.webapi import WebAPI
 
 if sys.version_info[0] >= 3:  # + 0.1 * sys.version_info[1] >= 3.3:
     PY3 = True
@@ -36,7 +36,7 @@ common_header = {
 }
 
 
-pcc.update(getConfig())
+pcc = getConfig()
 defaulturl = 'http://' + pcc['node']['host'] + \
     ':' + str(pcc['node']['port']) + pcc['baseurl']
 
@@ -56,40 +56,29 @@ def urn2fdiurl(urn, poolurl, contents='product', method='GET'):
     method:
     'GET' compo for retrieving product or hk or classes, urns, tags,
     'POST' compo for uploading  product
-    DELETE compo for removing product or pool
+    'PUT' for registering pool
+    'DELETE' compo for removing product or removing pool
 
     Example:
-    IP=ip poolpath=/a poolname=b/c files=/a/b/c/classes.jsn | urns.jsn | t.. | urn...
+    IP=ip poolpath=/a poolname=b files=/a/b/classes.jsn | urns.jsn | t.. | urn...
 
     with python:
     m.refs['myinput'] = special_ref
     ref=pstore.save(m)
-    assert ref.urn == 'urn:c:fdi.dataset.MapContext:203'
+    assert ref.urn == 'urn:b:fdi.dataset.MapContext:203'
     p=ref.product
     myref=p.refs['myinput']
 
     with a pool:
-    myref=pool.load('http://ip:port/v0.6/a/b/c/fdi.dataset.MapContext/203/refs/myinput')
+    myref=pool.load('http://ip:port/v0.6/b/fdi.dataset.MapContext/203/refs/myinput')
 
-    At the same time this is not allowed due to overlapping after 'c'
-
-    urn:http://ip:port/a/b/d/fdi.dataset.Product:203    ==>
-    http://ip:port/v0.6/a/b/c/d/
-
-    This is also overlapping starting from '/'
-
-    urn:http://ip:port/fdi.dataset.Product:203    ==>
-    http://ip:port/v0.6/
-
-    but for example this is allowed:
-    urn:http://ip:port/a/k/fdi.dataset.Product:203    ==>
-    http://ip:port/v0.6/a/k/
     """
 
-    poolname, resourcecn, index = parseUrn(urn)
+    poolname, resourcecn, index = parseUrn(
+        urn) if len(urn) > 7 else ('', '', '0')
     indexs = str(index)
     poolpath, scheme, place, pn = parse_poolurl(poolurl, poolhint=poolname)
-    if poolname is None:
+    if not poolname:
         poolname = pn
     if method == 'GET':
         if contents == 'product':
@@ -98,6 +87,8 @@ def urn2fdiurl(urn, poolurl, contents='product', method='GET'):
             ret = poolurl + '/hk'
         elif contents in ['classes', 'urns', 'tags']:
             ret = poolurl + '/hk/' + contents
+        elif contents.split('/')[0] in WebAPI:
+            ret = poolurl + '/api/' + contents
         else:
             raise ValueError(
                 'No such method and contents composition: ' + method + ' / ' + contents)
@@ -107,6 +98,12 @@ def urn2fdiurl(urn, poolurl, contents='product', method='GET'):
         else:
             raise ValueError(
                 'No such method and contents composition: ' + method + ' / ' + contents)
+    elif method == 'PUT':
+        if contents == 'pool':
+            ret = poolurl
+        else:
+            raise ValueError(
+                'No such method and contents composition: ' + method + ' / ' + contents)
     elif method == 'DELETE':
         if contents == 'pool':
             ret = poolurl
@@ -122,8 +119,14 @@ def urn2fdiurl(urn, poolurl, contents='product', method='GET'):
 # Store tag in headers, maybe that's  not a good idea
 
 
-def save_to_server(data, urn, poolurl, tag):
+def save_to_server(data, urn, poolurl, tag, no_serial=False):
     """Save product to server with putting tag in headers
+
+    data: goes to the request body
+    urn: to extract poolname, product type, and index if any of these are needed
+    poolurl: the only parameter must be provided
+    tag: go with the products into the pool
+    no_serial: do not serialize the data.
     """
     user = pcc['auth_user']
     password = pcc['auth_pass']
@@ -131,8 +134,9 @@ def save_to_server(data, urn, poolurl, tag):
     api = urn2fdiurl(urn, poolurl, contents='product', method='POST')
     # print('POST API: ' + api)
     headers = {'tag': tag}
+    sd = data if no_serial else serialize(data)
     res = requests.post(
-        api, auth=auth, data=serialize(data), headers=headers)
+        api, auth=auth, data=sd, headers=headers)
     result = deserialize(res.text)
     # print(result)
     return result
@@ -140,6 +144,9 @@ def save_to_server(data, urn, poolurl, tag):
 
 def read_from_server(urn, poolurl, contents='product'):
     """Read product or hk data from server
+
+    urn: to extract poolname, product type, and index if any of these are needed
+    poolurl: the only parameter must be provided
     """
     user = pcc['auth_user']
     password = pcc['auth_pass']
@@ -151,8 +158,27 @@ def read_from_server(urn, poolurl, contents='product'):
     return result['result'], result['msg']
 
 
+def put_on_server(urn, poolurl, contents='pool'):
+    """Register the pool on the server.
+
+    urn: to extract poolname, product type, and index if any of these are needed
+    poolurl: the only parameter must be provided
+    """
+    user = pcc['auth_user']
+    password = pcc['auth_pass']
+    auth = HTTPBasicAuth(user, password)
+    api = urn2fdiurl(urn, poolurl, contents=contents, method='PUT')
+    # print("DELETE REQUEST API: " + api)
+    res = requests.put(api, auth=auth)
+    result = deserialize(res.text)
+    return result['result'], result['msg']
+
+
 def delete_from_server(urn, poolurl, contents='product'):
     """Remove a product or pool from server
+
+    urn: to extract poolname, product type, and index if any of these are needed
+    poolurl: the only parameter must be provided
     """
     user = pcc['auth_user']
     password = pcc['auth_pass']
