@@ -1,7 +1,6 @@
 # -*- coding: utf-8 -*-
 
-from .server_skeleton import bad_request, unauthorized, not_found, conflict
-from ..utils.common import trbk, trbk2
+from ..utils.common import trbk, trbk2, getUidGid, lls
 from ..utils.run_proc import run_proc
 from ..dataset.deserialize import deserialize
 from ..dataset.serializable import serialize
@@ -19,28 +18,32 @@ from os.path import isfile, isdir, join
 from os import listdir, chown, chmod, environ, setuid, setgid
 from pathlib import Path
 from subprocess import Popen, PIPE, TimeoutExpired, run as srun
-from flask import abort, make_response, request
+from flask import abort, make_response, request, jsonify
 import filelock
 import sys
+import logging
 
 # from .logdict import logdict
 # '/var/log/pns-server.log'
 # logdict['handlers']['file']['filename'] = '/tmp/server.log'
 
 
-from .server_skeleton import setuplogging, checkpath, pc, Classes, app, auth, APIs, gid, getUidGid
+from .server_skeleton import init_conf_clas, makepublicAPI, checkpath, app
+from ..utils import getconfig
+pc = getconfig.getConfig()
 
-logging = setuplogging()
 logger = logging.getLogger(__name__)
 
-logger.setLevel(pc['logginglevel'])
-logger.debug('logging level %d' % (logger.getEffectiveLevel()))
-
 
 @app.before_first_request
 def init_pns_module():
-    global pc, gid
+    global Classes, pc
+
+    Classes = init_conf_clas(pc)
+    from ..dataset.classes import Classes
+
     ptsuid, ptsgid = getUidGid(pc['ptsuser'])
+    uid, gid = getUidGid(pc['ptsuser'])
     sgroup = os.getgrouplist(pc['serveruser'], gid)
     if ptsgid not in sgroup:
         logger.error('ptsuser %s must be in the group of serveruser %s %s.' %
@@ -74,16 +77,17 @@ def initPTS(d=None):
     """
 
     logger.debug(str(d))
+    checkpath.cache_clear()
 
     pnsh = pc['paths']['pnshome']
-    p = checkpath(pnsh)
+    p = checkpath(pnsh, pc['serveruser'])
     if p is None:
         abort(401)
 
-    pi = checkpath(pc['paths']['inputdir'])
+    pi = checkpath(pc['paths']['inputdir'], pc['serveruser'])
     if pi is None:
         abort(401)
-    po = checkpath(pc['paths']['outputdir'])
+    po = checkpath(pc['paths']['outputdir'], pc['serveruser'])
     if po is None:
         abort(401)
 
@@ -102,13 +106,15 @@ def testinit(d=None):
     """     Renames the 'init' 'config' 'run' 'clean' scripts to '.save' and points it to the '.ori' scripts.
     """
 
+    checkpath.cache_clear()
+
     pnsh = pc['paths']['pnshome']
-    p = checkpath(pnsh)
+    p = checkpath(pnsh, pc['serveruser'])
     if p is None:
         abort(401)
 
-    pi = checkpath(pc['paths']['inputdir'])
-    po = checkpath(pc['paths']['outputdir'])
+    pi = checkpath(pc['paths']['inputdir'], pc['serveruser'])
+    po = checkpath(pc['paths']['outputdir'], pc['serveruser'])
     if pi is None or po is None:
         abort(401)
 
@@ -259,12 +265,12 @@ def run(d, processinput=None, processoutput=None):
     global lupd
 
     pnsh = pc['paths']['pnshome']
-    p = checkpath(pnsh)
+    p = checkpath(pnsh, pc['serveruser'])
     if p is None:
         abort(401)
 
-    pi = checkpath(pc['paths']['inputdir'])
-    po = checkpath(pc['paths']['outputdir'])
+    pi = checkpath(pc['paths']['inputdir'], pc['serveruser'])
+    po = checkpath(pc['paths']['outputdir'], pc['serveruser'])
     if pi is None or po is None:
         abort(401)
 
@@ -351,12 +357,16 @@ def genposttestprod(d):
 
 def dosleep(indata, ops):
     """ run 'sleep [ops]' in the OS. ops is 3 if not given.
+
+    indata: possible {'timeout':xx}
     """
+    logger.debug('dosleep %s, %s' % (str(indata), str(ops)))
+
     try:
         t = float(ops)
     except ValueError as e:
         t = 3
-    if hasattr(indata, '__iter__') and 'timeout' in indata:
+    if hasattr(indata, 'items') and 'timeout' in indata:
         timeout = indata['timeout']
     else:
         timeout = pc['timeout']
@@ -381,7 +391,16 @@ def filesin(dir):
     return result, ''
 
 
-@app.route(pc['baseurl'] + '/<string:cmd>', methods=['GET'])
+@app.route(pc['baseurl'], methods=['GET'])
+def get_apis():
+    """ Makes a page for APIs described in module variable APIs. """
+
+    logger.debug('APIs %s' % (APIs.keys()))
+    l = [(a, makepublicAPI(o)) for a, o in APIs.items()]
+    return jsonify(dict(l))
+
+
+# @app.route(pc['baseurl'] + '/<string:cmd>', methods=['GET'])
 def getinfo(cmd):
     ''' returns init, config, run input, run output.
     '''
@@ -389,19 +408,26 @@ def getinfo(cmd):
 
     msg = ''
     ts = time.time()
+
     try:
-        if cmd == 'init':
+        if cmd == '' or cmd == 'api':
+            return get_apis()
+        elif cmd == 'init':
             with open(pc['scripts']['init'][0], 'r') as f:
                 result = f.read()
+            msg = "OK"
         elif cmd == 'config':
             with open(pc['scripts']['config'][0], 'r') as f:
                 result = f.read()
+            msg = "OK"
         elif cmd == 'run':
             with open(pc['scripts']['run'][0], 'r') as f:
                 result = f.read()
+            msg = "OK"
         elif cmd == 'clean':
             with open(pc['scripts']['clean'][0], 'r') as f:
                 result = f.read()
+            msg = "OK"
         elif cmd == 'input':
             result, msg = filesin(pc['paths']['inputdir'])
         elif cmd == 'output':
@@ -412,20 +438,10 @@ def getinfo(cmd):
             result, msg = -1, cmd + ' is not valid.'
     except Exception as e:
         result, msg = -1, str(e) + trbk(e)
-    w = {'result': result, 'message': msg, 'timestamp': ts}
-
-    s = serialize(w)
-    logger.debug(s[:] + ' ...')
-    resp = make_response(s)
-    resp.headers['Content-Type'] = 'application/json'
-    return resp
+    return result, msg
 
 
-@app.route(pc['baseurl'] + '/<string:cmd>', methods=['POST'])
-@app.route(pc['baseurl'] + '/<string:cmd>/<string:ops>', methods=['POST'])
 def calcresult(cmd, ops=''):
-
-    logger.debug('POST ' + cmd + ' ' + ops)
     d = request.get_data()
     if cmd == 'calc':
         result, msg = calc(d)
@@ -471,11 +487,32 @@ def calcresult(cmd, ops=''):
                 logger.error(cmd)
                 abort(400)
                 result = None
-    logger.debug(str(result)[:155])
+                msg = lls(result, 155)
+    return result, msg
+
+
+@app.route(pc['baseurl'] + '/<string:cmd>', methods=['GET', 'POST'])
+@app.route(pc['baseurl'] + '/<string:cmd>/<string:ops>', methods=['POST'])
+def get_post(cmd, ops=''):
+    """ get and post.
+
+    GET: returns init, config, run input, run output.
+    POST:
+    """
+
+    logger.debug(request.method + ' ' + cmd + ' ' + ops)
+    if request.method == 'POST':
+        result, msg = calcresult(cmd, ops=ops)
+    elif request.method == 'GET':
+        if cmd in ['', '/', 'api']:
+            return getinfo(cmd)
+        result, msg = getinfo(cmd)
+    else:
+        raise RuntimeError('strange ' + request.method)
     ts = time.time()
     w = {'result': result, 'message': msg, 'timestamp': ts}
     s = serialize(w)
-    logger.debug(s[: 160] + ' ...')
+    logger.debug('serialized result: '+lls(s, 160))
     resp = make_response(s)
     resp.headers['Content-Type'] = 'application/json'
     return resp
@@ -501,6 +538,7 @@ def setup(cmd, ops=''):
         logger.debug('busy')
         abort(409, 'pns is busy')
     with lock:
+        checkpath.cache_clear()
         if cmd == 'init':
             try:
                 result, msg = initPTS(d)
@@ -526,7 +564,6 @@ def setup(cmd, ops=''):
     ts = time.time()
     w = {'result': result, 'message': msg, 'timestamp': ts}
     s = serialize(w)
-    # logger.debug(s[:] + ' ...')
     resp = make_response(s)
     resp.headers['Content-Type'] = 'application/json'
     return resp
@@ -554,38 +591,36 @@ def cleanup(cmd):
     ts = time.time()
     w = {'result': result, 'message': msg, 'timestamp': ts}
     s = serialize(w)
-    logger.debug(s[:] + ' ...')
+    logger.debug(s)
     resp = make_response(s)
     resp.headers['Content-Type'] = 'application/json'
     return resp
 
 
 # API specification for this module
-ModAPIs = {'GET':
-           {'func': 'getinfo',
-            'cmds': {'init': 'the initPTS file', 'config': 'the configPTS file',
-                     'run': 'the file running PTS', 'clean': 'the cleanPTS file',
-                     'input': filesin, 'output': filesin,
-                     'pnsconfig': 'PNS configuration'}
-            },
-           'PUT':
-           {'func': 'setup',
-               'cmds': {'init': initPTS, 'config': configPTS, 'pnsconf': configPNS, 'testinit': testinit}
-            },
-           'POST':
-           {'func': 'calcresult',
-               'cmds': {'calc': calc, 'testcalc': genposttestprod,
-                        'run': run, 'testrun': testrun,
-                        'echo': 'Echo',
-                        'sleep': (dosleep,  dict(ops='3'))}
-            },
-           'DELETE':
-           {'func': 'cleanup',
-               'cmds': {'clean': cleanPTS}
-            }}
+APIs = {'GET':
+        {'func': 'get_post',
+         'cmds': {'': get_apis, 'api': get_apis,
+                  'init': 'the initPTS file', 'config': 'the configPTS file',
+                  'run': 'the file running PTS', 'clean': 'the cleanPTS file',
+                  'input': filesin, 'output': filesin,
+                  'pnsconfig': 'PNS configuration'}
+         },
+        'PUT':
+        {'func': 'setup',
+         'cmds': {'init': initPTS, 'config': configPTS, 'pnsconf': configPNS, 'testinit': testinit}
+         },
+        'POST':
+        {'func': 'get_post',
+         'cmds': {'calc': calc, 'testcalc': genposttestprod,
+                  'run': run, 'testrun': testrun,
+                  'echo': 'Echo',
+                  'sleep': (dosleep,  dict(ops='3'))}
+         },
+        'DELETE':
+        {'func': 'cleanup',
+         'cmds': {'clean': cleanPTS}
+         }}
 
 
-# Use ModAPIs contents for server_skeleton.get_apis()
-APIs.update(ModAPIs)
-
 logger.debug('END OF '+__file__)
