@@ -2,62 +2,107 @@
 from os.path import join
 import logging
 import getpass
-import pwd
+import os
 
-# logging level for server or possibly by client
-pnsconfig = dict(logginglevel=logging.DEBUG)
+pnsconfig = {}
+
+# look-up table for PoolManager (therefor HttpClient) to get pool URLs eith Pool ID (poolname)
+poolurl_of = {
+    'svom': 'http://10.0.10.114:9885/v0.6/svom'
+}
+pnsconfig['lookup'] = poolurl_of
+
+###########################################
+# Configuration for Servers running locally.
+# components of the default poolurl
+
+# the key (variable names) must be uppercased for Flask server
+# FLASK_CONF = pnsconfig
+
+# Te be edited automatically with sed -i 's/^EXTHOST =.*$/EXTHOST = xxx/g' file
+EXTHOST = '172.17.0.1'
+EXTPORT = 9876
 
 # base url for webserver. Update version if needed.
-pnsconfig['api_version'] = 'v0.6'
+pnsconfig['api_version'] = 'v0.8'
 pnsconfig['baseurl'] = '/' + pnsconfig['api_version']
 
-# base url for pool, you must have permission of this path, for example : /home/user/Documents
-# this base pool path will be added at the beginning of your pool urn when you init a pool like:
-# pstore = PoolManager.getPool('/demopool_user'), it will create a pool at pc['base_poolpath']/demopool_user/
-# User can disable  basepoolpath by: pstore = PoolManager.getPool('/demopool_user', use_default_poolpath=False). Also note that pool URL takes priority if given to getPool().
+
+# default path of working directories for LocalPools.
 pnsconfig['base_poolpath'] = '/tmp'
+# default path of working directories for HttpPool server.
 pnsconfig['server_poolpath'] = '/var/www/httppool_server/data'  # For server
+# you must have write permission of above paths.
+# For example : /home/user/Documents
+# will be added at the beginning of request pool ID 'my_pool'
+# when you init a server pool like:
+# ``pstore = PoolManager.getPool('my_pool') without poolurl,
+# the poolurl will become 'file:///home/user/Documents/my_pool'.
+# creating a pool at pc['base_poolpath']
+# See document of :meth:'PoolManager.getPool'
+
+# logging level for server
+pnsconfig['logginglevel'] = logging.DEBUG
+# for HttpPool
 pnsconfig['defaultpool'] = 'pool_default'
 
-dev = 0
-if dev:
+# choose from pre-defined.
+conf = ['dev', 'server_test', 'external'][0]
+
+# modify
+if conf == 'dev':
     # username, passwd, flask ip, flask port
     pnsconfig['node'] = {'username': 'foo',
                          'password': 'bar', 'host': '0.0.0.0', 'port': 5000}
 
     # server permission user
     pnsconfig['serveruser'] = 'mh'
-    # PTS app permission user
-    pnsconfig['ptsuser'] = 'mh'
-    # on server
-    home = '/cygdrive/c/Users/mh'
     pnsconfig['base_poolpath'] = '/tmp'
     pnsconfig['server_poolpath'] = '/tmp/data'  # For server
-    pnsconfig['defaultpool'] = 'pool_default'
-else:
+    # on pns server
+    home = '/cygdrive/c/Users/mh'
+    # PTS app permission user
+    pnsconfig['ptsuser'] = 'mh'
+
+elif conf == 'server_test':
     pnsconfig['node'] = {'username': 'foo', 'password': 'bar',
-                         'host': '127.0.0.1', 'port': 9884}
+                         'host': '172.0.17.9', 'port': 9884}
+    # server permission user
+    pnsconfig['serveruser'] = 'apache'
+    # PTS app permission user
+    pnsconfig['ptsuser'] = 'pns'
+    # on pns server
+    home = '/home'
+elif conf == 'external':
+    # wsgi behind apach2. cannot use env vars
+    pnsconfig['node'] = {'username': 'foo', 'password': 'bar',
+                         'host': EXTHOST, 'port': EXTPORT}
 
     # server permission user
     pnsconfig['serveruser'] = 'apache'
     # PTS app permission user
     pnsconfig['ptsuser'] = 'pns'
-    # on server
+    # on pns server
     home = '/home'
-
+else:
+    pass
 pnsconfig['auth_user'] = pnsconfig['node']['username']
 pnsconfig['auth_pass'] = pnsconfig['node']['password']
+# It is best not use this 'httphost' in apps, and use url_of through getConfig
 pnsconfig['httphost'] = 'http://' + \
     pnsconfig['node']['host']+':'+str(pnsconfig['node']['port'])
-pnsconfig['poolprefix'] = pnsconfig['httphost']
 pnsconfig['mysql'] = {'host': 'ssa-mysql', 'port': 3306,
                       'user': 'root',  'password': '123456',
                       'database': 'users'}
 
-# import user classes
-# '/cygdrive/d/code/share/svom/products/projectclasses.py'
+
+# import user classes for server.
+# See document in :class:`Classes`
 pnsconfig['userclasses'] = ''
 
+
+########### PNS-specific setup ############
+
 phome = join(home, 'pns')
 pnsconfig['paths'] = dict(
     pnshome=phome,
