@@ -1,26 +1,32 @@
 #!flask/bin/python
 # -*- coding: utf-8 -*-
 
-from fdi.pns.pnsconfig import pnsconfig as pc
 from fdi.utils.options import opt
-from fdi.utils.getconfig import getConfig
+from fdi.utils import getconfig
+
+import logging
+import sys
 
-from waitress import serve
 #sys.path.insert(0, abspath(join(join(dirname(__file__), '..'), '..')))
 
 # print(sys.path)
 
 
-def setuplogging():
-    import logging.config
-    import logging
-    from fdi.pns import logdict  # import logdict
+def setuplogging(level=logging.WARN):
+    global logging
     # create logger
-    logging.config.dictConfig(logdict.logdict)
+    logging.basicConfig(stream=sys.stdout,
+                        format='%(asctime)s-%(levelname)4s'
+                        '-[%(filename)6s:%(lineno)3s'
+                        '-%(funcName)10s()] - %(message)s',
+                        datefmt="%Y%m%d %H:%M:%S")
+    logging.getLogger("requests").setLevel(level)
+    logging.getLogger("filelock").setLevel(level)
+    if sys.version_info[0] > 2:
+        logging.getLogger("urllib3").setLevel(level)
     return logging
 
 
-logging = setuplogging()
 logger = logging.getLogger(__name__)
 
 
@@ -28,9 +34,13 @@ if __name__ == '__main__':
 
     logger = logging.getLogger()
     # default configuration is provided. Copy pnsconfig.py to ~/.config/pnslocal.py
-    pc.update(getConfig())
-    logger.setLevel(pc['logginglevel'])
-    logger.info('Server starting. Make sure no other instance is running')
+    pc = getconfig.getConfig()
+
+    lv = pc['logginglevel']
+    logger.setLevel(lv)
+    setuplogging(lv if lv > logging.WARN else logging.WARN)
+    logger.info(
+        'Server starting. Make sure no other instance is running.'+str(lv))
 
     node = pc['node']
     # Get username and password and host ip and port.
@@ -82,7 +92,8 @@ if __name__ == '__main__':
         sys.exit(-1)
 
     if wsgi:
+        from waitress import serve
         serve(app, url_scheme='https', host=node['host'], port=node['port'])
     else:
         app.run(host=node['host'], port=node['port'],
-                threaded=True, debug=verbose, processes=1, use_reloader=False)
+                threaded=True, debug=verbose, processes=1, use_reloader=True)
