@@ -182,9 +182,9 @@ nds3 =\
 #=== dimension 4
 
 """
+
 out_GenericDataset = """level 0
-# GenericDataset
-description= {'test GD'},
+=== GenericDataset () ===
 meta= {
 ======  =================  ======  ========  =======================  ===============  ======  =====================
 name    value              unit    type      valid                    default          code    description
@@ -205,8 +205,7 @@ MetaData-listeners = ListnerSet{}}
 GenericDataset-dataset =
 88.8
 level 1, repr
-# GenericDataset
-description= {'test GD'},
+=== GenericDataset () ===
 meta= {
 -------------  -----------------  ----------------
 a= 3.4         b= xy (2019-02-19  c= Invalid (IJK)
@@ -221,8 +220,7 @@ GenericDataset-dataset =
 level 2,
 GenericDataset{ 88.8, description = "test GD", meta = a, b, c, d, listeners = ListnerSet{} }"""
 out_ArrayDataset = """level 0
-# ArrayDataset
-description= {'toString tester AD'},
+=== ArrayDataset () ===
 meta= {
 ======  =================  ======  ========  =======================  ===============  ======  =====================
 name    value              unit    type      valid                    default          code    description
@@ -241,9 +239,9 @@ d       off (0b00)         None    binary    11000 0b01: on           0b0
 ======  =================  ======  ========  =======================  ===============  ======  =====================
 MetaData-listeners = ListnerSet{}},
 type= {None},
+unit= {'lyr'},
 default= {None},
-typecode= {None},
-unit= {'lyr'}
+typecode= {None}
 ArrayDataset-dataset =
 0  0  0  0  0
 0  0  0  0  0
@@ -287,8 +285,7 @@ ArrayDataset-dataset =
 
 
 level 1, repr
-# ArrayDataset
-description= {'toString tester AD'},
+=== ArrayDataset () ===
 meta= {
 -------------  -----------------  ----------------
 a= 3.4         b= xy (2019-02-19  c= Invalid (IJK)
@@ -299,9 +296,9 @@ d= off (0b00)
 MetaData-listeners = ListnerSet{}
 },
 type= {None},
+unit= {'lyr'},
 default= {None},
-typecode= {None},
-unit= {'lyr'}
+typecode= {None}
 ArrayDataset-dataset =
 0  0  0  0  0
 0  0  0  0  0
@@ -345,17 +342,67 @@ ArrayDataset-dataset =
 
 
 level 2,
-ArrayDataset{ [[[[0, 0, 0, ...0]]], [[[0, 0, 0, ...0]]]] (lyr) <None>, "toString tester AD", default None, tcode=None, meta=
--------------  -----------------  ----------------
-a= 3.4         b= xy (2019-02-19  c= Invalid (IJK)
-               01:02:03.456789
-               1929229323456789)
-d= off (0b00)
--------------  -----------------  ----------------
-MetaData-listeners = ListnerSet{}}"""
+ArrayDataset{ [[[[0, 0, 0, ...0]]], [[[0, 0, 0, ...0]]]] (lyr) <None>, "toString tester AD", default None, tcode=None, meta=_sets:
+      a: !NumericParameter
+            description: rule name, if is "valid", "", or "default", is ommited in
+                  value string.
+            value: 3.4
+            type: float
+            default: 2.0
+            valid:
+              -   -   - 0
+                      - 31
+                  - valid
+              -   - 99
+                  - ''
+            unit:
+            typecode:
+            _STID: NumericParameter
+      b: !DateParameter
+            description: date param
+            value: !FineTime
+                  tai: 1929229323456789
+                  format: '%Y-%m-%dT%H:%M:%S.%f UTC'
+                  _STID: FineTime
+            default: !FineTime
+                  tai: 99
+                  format: '%Y-%m-%dT%H:%M:%S.%f UTC'
+                  _STID: FineTime
+            valid:
+              -   -   - 0
+                      - 9876543210123456
+                  - xy
+            typecode: '%Y-%m-%dT%H:%M:%S.%f UTC'
+            _STID: DateParameter
+      c: !StringParameter
+            value: IJK
+            description: str parameter. but only "" is allowed.
+            valid:
+              -   - ''
+                  - empty
+            default: cliche
+            typecode: B
+            _STID: StringParameter
+      d: !NumericParameter
+            description: valid rules described with binary masks
+            value: 1
+            type: binary
+            default: 0
+            valid:
+              -   -   - 24
+                      - 1
+                  - on
+              -   -   - 24
+                      - 0
+                  - off
+            unit:
+            typecode:
+            _STID: NumericParameter
+listeners: !ListnerSet {}
+_STID: MetaData
+}"""
 out_TableDataset = """level 0
-# TableDataset
-description= {'UNKNOWN'},
+=== TableDataset () ===
 meta= {
 ======  =================  ======  ========  =======================  ===============  ======  =====================
 name    value              unit    type      valid                    default          code    description
@@ -383,8 +430,7 @@ TableDataset-dataset =
 
 
 level 1, repr
-# TableDataset
-description= {'UNKNOWN'},
+=== TableDataset () ===
 meta= {
 -------------  -----------------  ----------------
 a= 3.4         b= xy (2019-02-19  c= Invalid (IJK)
@@ -404,8 +450,8 @@ TableDataset-dataset =
 
 
 level 2,
-# TableDataset
-description, meta
+=== TableDataset () ===
+meta
 TableDataset-dataset =
   col1     col2
   (eV)    (cnt)
@@ -416,8 +462,7 @@ TableDataset-dataset =
 (Only display 2 rows of 3 for level=2.)
 """
 out_CompositeDataset = """level 0
-# CompositeDataset
-description= {'test CD'},
+=== CompositeDataset () ===
 meta= {
 ======  =================  ======  ========  =======================  ===============  ======  =====================
 name    value              unit    type      valid                    default          code    description
@@ -439,18 +484,18 @@ m1      2.3                sec     float     None                     None
 MetaData-listeners = ListnerSet{}}
 
 CompositeDataset-datasets =
-<ODict     [ dataset 1 ]= # ArrayDataset
-description= {'arraydset 1'},
-meta= {(No parameter.)MetaData-listeners = ListnerSet{}},
+<ODict "dataset 1":
+=== ArrayDataset () ===
+meta= {(No Parameter.) MetaData-listeners = ListnerSet{}},
 type= {None},
+unit= {'ev'},
 default= {None},
-typecode= {None},
-unit= {'ev'}
+typecode= {None}
 ArrayDataset-dataset =
 768  4.4  5400
-    [ dataset 2 ]= # TableDataset
-description= {'Example table'},
-meta= {(No parameter.)MetaData-listeners = ListnerSet{}}
+"dataset 2":
+=== TableDataset () ===
+meta= {(No Parameter.) MetaData-listeners = ListnerSet{}}
 TableDataset-dataset =
    Time    Energy
   (sec)      (eV)
@@ -463,8 +508,7 @@ TableDataset-dataset =
 
 
 >level 1, repr
-# CompositeDataset
-description= {'test CD'},
+=== CompositeDataset () ===
 meta= {
 -------------  -----------------  ----------------
 a= 3.4         b= xy (2019-02-19  c= Invalid (IJK)
@@ -476,19 +520,17 @@ MetaData-listeners = ListnerSet{}
 }
 
 CompositeDataset-datasets =
-<ODict     [ dataset 1 ]= # ArrayDataset
-description= {'arraydset 1'},
-meta= {(No parameter.)MetaData-listeners = ListnerSet{}
+<ODict  === ArrayDataset () ===
+meta= {(No Parameter.) MetaData-listeners = ListnerSet{}
 },
 type= {None},
+unit= {'ev'},
 default= {None},
-typecode= {None},
-unit= {'ev'}
+typecode= {None}
 ArrayDataset-dataset =
 768  4.4  5400
-    [ dataset 2 ]= # TableDataset
-description= {'Example table'},
-meta= {(No parameter.)MetaData-listeners = ListnerSet{}
+ === TableDataset () ===
+meta= {(No Parameter.) MetaData-listeners = ListnerSet{}
 }
 TableDataset-dataset =
    Time    Energy
@@ -502,12 +544,15 @@ TableDataset-dataset =
 
 
 >level 2,
-# CompositeDataset
-description, meta
+=== CompositeDataset () ===
+meta
 
 CompositeDataset-datasets =
-<ODict     [ dataset 1 ]= ArrayDataset{ [768, 4.4, 5400.0] (ev) <None>, "arraydset 1", default None, tcode=None, meta=(No parameter.)MetaData-listeners = ListnerSet{}}    [ dataset 2 ]= # TableDataset
-description, meta
+<ODict  ArrayDataset{ [768, 4.4, 5400.0] (ev) <None>, "arraydset 1", default None, tcode=None, meta=_sets: {}
+listeners: !ListnerSet {}
+_STID: MetaData
+} === TableDataset () ===
+meta
 TableDataset-dataset =
    Time    Energy
   (sec)      (eV)
