@@ -9,6 +9,7 @@ import copy
 import sys
 import functools
 import array
+from math import sqrt
 from datetime import timezone
 import pytest
 
@@ -49,7 +50,7 @@ else:
 
 Classes.updateMapping()
 
-# make format output in /tmp/fditest_toString
+# make format output in /tmp/output.py
 mko = 0
 
 if __name__ == '__main__' and __package__ is None:
@@ -154,6 +155,8 @@ def test_serialization():
     checkjson(v, 1)
     v = [1.2, 'ww']
     checkjson(v)
+    v = Product
+    checkjson(v)
     # v = (1, 8.2, 'tt')
     # checkjson(v)
     # v = {(5, 4): 4, 'y': {('d', 60): 'ff', '%': '$'}}
@@ -441,9 +444,14 @@ def test_datatypes():
     assert v.getComponents() == [0, 0]
     v = Vector([1, 2.3])
     assert v.getComponents() == [1, 2.3]
+    # comparison
+    assert v > sqrt(1+2.3**2) - 1
+    assert v < sqrt(1+2.3**2) + 1
+
     # assignment
     v.components = [0xaa, 1, 1e2]
     assert v.components == [0xaa, 1, 1e2]
+    h = hash(v)
     checkjson(v)
 
     # Quaternion
@@ -453,6 +461,7 @@ def test_datatypes():
     a1 = -1
     v2 = Quaternion([a1, 1+0, 1-a1+0.3, 4.5])
     assert v == v2
+    h = hash(v)
     checkjson(v)
 
 
@@ -701,7 +710,11 @@ def test_Parameter_features():
     b1 = ''.join(a1)  # make a new string copy
     b2 = a2 + 0  # make a copy
     v1 = Parameter(description=b1, value=b2)
+
+    # CHANNGED parameter equality behavior
     assert v.equals(v1)
+    assert v.__eq__(v1)
+    assert v.__eq__ == v1.__eq__
     assert v == v1
     v1.value = -4
 
@@ -710,8 +723,12 @@ def test_Parameter_features():
     b2 = a2 + 0  # make a copy
     v1.value = b2  # change it back
     v1.description = 'changed'
-    assert not v.equals(v1)
-    assert v != v1
+    if 0:
+        assert not v.equals(v1)
+        assert v != v1
+    else:
+        assert v.equals(v1)
+        assert v == v1
 
     # comparison with simplified syntax w/o '.value'
     x = 4
@@ -861,6 +878,7 @@ def test_NumericParameter():
     assert v.default is None
     assert v.valid is None
     assert v.typecode is None
+    h = v.hash()
 
     a1 = 'a test NumericParameter'
     a2 = 100.234
@@ -914,7 +932,7 @@ def test_DateParameter():
 
     with pytest.raises(TypeError):
         DateParameter(3.3)
-
+    vv = copy.copy(v)
     checkjson(v)
 
 
@@ -979,7 +997,8 @@ def test_MetaData():
     # names of all parameters
     assert [n for n in v] == [a1, 'time', 'birthday']
 
-    checkjson(v, dbg=1)
+    h = v.hash()
+    checkjson(v, dbg=0)
 
     v.remove(a1)  # inherited from composite
     assert v.size() == 2
@@ -1096,7 +1115,7 @@ def test_GenericDataset():
     ts += v.toString(2)
     if mko:
         print(ts)
-        with open('/tmp/fditest_toString', 'wt') as f:
+        with open('/tmp/output.py', 'wt') as f:
             clsn = 'out_GenericDataset'
             f.write('%s = """%s"""\n' % (clsn, ts))
     else:
@@ -1210,7 +1229,7 @@ def do_ArrayDataset_func(atype):
     # v and v1 are not the same, really
     assert id(v) != id(v1)
     # change meta
-    v1.meta[b4].description = 'c'
+    v1.meta[b4].value = 999
     assert v != v1
     assert v1 != v
 
@@ -1294,12 +1313,14 @@ def do_ArrayDataset_func(atype):
     ts += x.toString(2)
     if mko:
         print(ts)
-        with open('/tmp/fditest_toString', 'a') as f:
+        with open('/tmp/output.py', 'a') as f:
             clsn = 'out_ArrayDataset'
             f.write('%s = """%s"""\n' % (clsn, ts))
     else:
         assert ts == out_ArrayDataset
 
+    h = hash(v)
+
     checkjson(x)
     checkgeneral(x)
 
@@ -1324,6 +1345,13 @@ def test_ArrayDataset_array_func():
     do_ArrayDataset_func(atype)
 
 
+def test_Column():
+    v = Column(data=[4, 9], unit='m')
+    s = v.hash()
+
+    checkjson(v)
+
+
 def test_TableModel():
     pass
 
@@ -1358,7 +1386,7 @@ def test_TableDataset_init():
     v3 = TableDataset(data=[('col1', [1, 4.4, 5.4E3], 'eV'),
                             ('col2', [0, 43.2, 2E3], 'cnt')
                             ])
-    assert v == v3  # , deepcmp(v, v3)
+    assert v == v3, deepcmp(v, v3)
 
     # 4: quick and dirty. data are list of lists without names or units
     a5 = [[1, 4.4, 5.4E3], [0, 43.2, 2E3]]
@@ -1566,7 +1594,7 @@ def test_TableDataset_func():
     ts += v.toString(2)
     if mko:
         print(ts)
-        with open('/tmp/fditest_toString', 'a') as f:
+        with open('/tmp/output.py', 'a') as f:
             clsn = 'out_TableDataset'
             f.write('%s = """%s"""\n' % (clsn, ts))
     else:
@@ -1706,10 +1734,6 @@ def demo_TableDataset():
     print(table["Time"].unit)
 
 
-def test_Column():
-    pass
-
-
 def test_CompositeDataset_init():
     # constructor
     a1 = [768, 4.4, 5.4E3]
@@ -1775,7 +1799,7 @@ def test_CompositeDataset_init():
     b4 = copy.deepcopy(a4)
     v1[b9] = b4
     assert v == v1
-    v1.meta[b11].description = 'c'
+    v1.meta[b11].value = 999
     assert v != v1
     assert v1 != v
 
@@ -1806,7 +1830,7 @@ def test_CompositeDataset_init():
     ts += v3.toString(2)
     if mko:
         print(ts)
-        with open('/tmp/fditest_toString', 'a') as f:
+        with open('/tmp/output.py', 'a') as f:
             clsn = 'out_CompositeDataset'
             f.write('%s = """%s"""\n' % (clsn, ts))
     else:
@@ -2049,6 +2073,7 @@ def test_History():
 
 def test_BaseProduct():
     """ """
+
     x = BaseProduct(description="This is my product example")
     # print(x.__dict__)
     # print(x.meta.toString())
