@@ -1,4 +1,4 @@
-from fdi.pns.pnsconfig import pnsconfig as pc
+
 from fdi.dataset.dataset import ArrayDataset
 import itertools
 import random
@@ -14,6 +14,7 @@ from fdi.pal.productstorage import ProductStorage
 from fdi.pal.urn import Urn, parseUrn, parse_poolurl, makeUrn, UrnUtils
 from fdi.pal.productpool import ProductPool
 from fdi.pal.localpool import LocalPool
+from fdi.pal.httpclientpool import HttpClientPool
 from fdi.pal.context import Context
 from fdi.pal.query import AbstractQuery, MetaQuery
 from fdi.dataset.deserialize import deserialize
@@ -24,8 +25,9 @@ from fdi.dataset.metadata import MetaData, Parameter
 from fdi.dataset.finetime import FineTime1
 from fdi.dataset.testproducts import TP
 from fdi.utils.checkjson import checkjson
-from fdi.utils.getconfig import getConfig
 from fdi.pns.fdi_requests import save_to_server, read_from_server, delete_from_server
+
+from requests.auth import HTTPBasicAuth
 import copy
 import traceback
 from pprint import pprint
@@ -34,7 +36,8 @@ import shutil
 import getpass
 import os
 import requests
-
+import weakref
+import gc
 from os import path as op
 import glob
 
@@ -47,7 +50,6 @@ else:
     PY3 = False
 
 Classes.updateMapping()
-pc.update(getConfig())
 
 if __name__ == '__main__' and __package__ == 'tests':
     # run by python -m tests.test_dataset
@@ -109,8 +111,8 @@ def test_UrnUtils():
     assert UrnUtils.getClass(urn).__name__ == a4.split('.')[-1]
     assert UrnUtils.extractRecordIDs([urn, urn1]) == [a5, a5-1]
 
-    #assert UrnUtils.getPool(urn,pools)
-    #assert UrnUtils.containsUrn(urn, pool)
+    # assert UrnUtils.getPool(urn,pools)
+    # assert UrnUtils.containsUrn(urn, pool)
     assert UrnUtils.checkUrn(urn)
     with pytest.raises(ValueError):
         UrnUtils.checkUrn(urn+'r')
@@ -169,6 +171,8 @@ def test_Urn():
     assert v.getScheme() is None
     assert v.getPlace() is None
     assert v.getPoolpath() is None
+
+    h = v.hash()
     # urn with poolurl
     v = Urn(urn=urn, poolurl=poolurl)
     assert v.getPoolId() == b2  #
@@ -254,17 +258,22 @@ def cleanup(poolurl=None, poolname=None):
         elif schm == 'mem':
             pass
         elif schm in ['http', 'https']:
-            d = PoolManager.PlacePaths['file'] + '/' + pn
-            rmlocal(d)
-            #
-            res, msg = delete_from_server(None, purl, 'pool')
-            assert res != 'FAILED'
-
+            pass
         else:
             assert False
+        # correct way
+        try:
+            p = PoolManager.getPool(pname, poolurl)
+            p.removeAll()
+        except RuntimeError:
+            pass
 
-        PoolManager.getPool(pn).removeAll()
-        del PoolManager.getMap()[pn]
+        PoolManager.remove(pname)
+        try:
+            del p
+        except NameError:
+            pass
+        gc.collect()
 
 
 def test_PoolManager():
@@ -276,29 +285,64 @@ def test_PoolManager():
 
     assert PoolManager.size() == 0
     # This creates a pool and returns it if the pool of given name does not exist
-    pool = PoolManager.getPool(defaultpoolName)
+    pool = PoolManager.getPool(defaultpoolName, defaultpoolUrl)
     assert PoolManager.size() == 1
     assert defaultpoolName in PoolManager.getMap()
     # print('GlobalPoolList#: ' + str(id(pm.getMap())) + str(pm))
     PoolManager.removeAll()
     assert PoolManager.size() == 0
+    assert weakref.getweakrefcount(pool) == 0
+    #print(weakref.getweakrefs(pool), id(pool), 'mmmm pool')
+    del pool
 
     # initiate
     pm = PoolManager()
     assert len(pm) == 0
-    pm.getPool(defaultpoolName)
+    p1 = pm.getPool(defaultpoolName, defaultpoolUrl)
+    #print(weakref.getweakrefs(p1), id(p1), 'mmmm p1')
     for k, v in pm.items():
         assert isinstance(v, ProductPool)
     assert defaultpoolName in pm
+    PG = PoolManager._GlobalPoolList
+    assert PoolManager.isLoaded(defaultpoolName) == 1
+    del v
+    assert PoolManager.isLoaded(defaultpoolName) == 1
+    # print(weakref.getweakrefs(PG[defaultpoolName]), id(
+    #    PG[defaultpoolName]), 'mmm GL')
+    pr = weakref.ref(p1)
+    assert pr() == p1
+    assert weakref.getweakrefcount(pr()) == 2
+    # print(weakref.getweakrefs(PG[defaultpoolName]),
+    #      id(PG[defaultpoolName]), 'mmm+r')
+    del p1
+    assert pr()
+    # print(weakref.getweakrefs(PG[defaultpoolName]),
+    #      id(PG[defaultpoolName]), 'mmmm del p1')
+    assert weakref.getweakrefcount(PG[defaultpoolName]) == 2
+    gc.collect()
+    # print(weakref.getweakrefs(PG[defaultpoolName]),
+    #      id(PG[defaultpoolName]), 'mmmm gc')
+    assert weakref.getweakrefcount(PG[defaultpoolName]) == 2  # why ?
+    del pr
+    # print(weakref.getweakrefs(PG[defaultpoolName]),
+    #      id(PG[defaultpoolName]), 'mmmm del r')
+    assert weakref.getweakrefcount(PG[defaultpoolName]) == 1
+    assert pm.remove(defaultpoolName) == 0
+
+    # http pool gets registered
+    with pytest.raises(RuntimeError):
+        ph = pm.getPool(poolurl='http://h.edu/foo')
+    assert not PoolManager.isLoaded('foo')
+    assert PoolManager.remove('foo') == 1
 
 
-def checkdbcount(expected_cnt, poolurl, prodname, currentSN=-1):
+def checkdbcount(expected_cnt, poolurl, prodname, currentSN, usrpsw, *args):
     """ count files in pool and entries in class db.
 
     expected_cnt, currentSN: expected number of prods and currentSN in pool for products named prodname
     """
-
-    poolpath, scheme, place, poolname = parse_poolurl(poolurl)
+    poolpath, schm, place, poolname = parse_poolurl(poolurl)
+    scheme = schm.lower()
     if scheme in ['file', 'server']:
         path = op.join(poolpath, poolname)
         assert sum(1 for x in glob.glob(
@@ -326,12 +370,13 @@ def checkdbcount(expected_cnt, poolurl, prodname, currentSN=-1):
         # for this class there are  how many prods
         assert len(mpool['classes'][prodname]['sn']) == expected_cnt
     elif scheme in ['http', 'https']:
-        snpath = 'sn/' + prodname + '/' + poolname
+        auth = HTTPBasicAuth(*usrpsw)
+        cpath = poolname + '/' + 'count/' + prodname
         api_baseurl = scheme + '://' + place + poolpath + '/'
-        url = api_baseurl + snpath
-        x = requests.get(url)
-        sn = int(x.text)
-        assert sn == expected_cnt
+        url = api_baseurl + cpath
+        x = requests.get(url, auth=auth)
+        count = int(x.json()['result'])
+        assert count == expected_cnt
     else:
         assert False, 'bad pool scheme'
 
@@ -357,7 +402,7 @@ def test_ProductRef():
     # A productref created from a single product will result in a memory pool urn, and the metadata won't be loaded.
     v = ProductRef(prd)
     # only one prod in memory pool
-    checkdbcount(1, 'mem:///'+DEFAULT_MEM_POOL, a4, 0)
+    checkdbcount(1, 'mem:///'+DEFAULT_MEM_POOL, a4, 0, '')
     assert v.urn == 'urn:'+DEFAULT_MEM_POOL+':' + a4 + ':' + str(0)
     assert v.meta is None
     assert v.product == prd
@@ -398,6 +443,7 @@ def test_ProductRef():
     pr.addParent(b2)
     assert b1 in list(pr.parents)
     assert b2 in list(pr.parents)
+
     pr.removeParent(b1)
     assert b1 not in list(pr.parents)
     # access
@@ -419,16 +465,18 @@ def test_ProductStorage_init():
     cleanup(newpoolurl, newpoolname)
 
     # Constructor
-    # default pool
+    # no default pool
     ps = ProductStorage()
-    p1 = ps.getPools()[0]
-    # check default pool's name
-    assert p1 == defaultpoolname
+    with pytest.raises(IndexError):
+        p1 = ps.getPools()[0]
+    # with poolurl the pool is constructed
+    ps = ProductStorage(poolurl=defaultpoolurl)
     # get the pool object
-    pspool = ps.getPool(p1)
+    pspool = ps.getPool(defaultpoolname)
     assert len(pspool.getProductClasses()) == 0
-    # check syntax: construct a storage with a pool
-    ps2 = ProductStorage(defaultpoolname)
+    # check syntax: construct a storage with a poolobj
+    pool = LocalPool(poolurl=defaultpoolurl)
+    ps2 = ProductStorage(pool)
     assert ps.getPools() == ps2.getPools()
     # wrong poolname
     with pytest.raises(TypeError):
@@ -437,7 +485,7 @@ def test_ProductStorage_init():
     # register pool
     # with a storage that already has a pool
 
-    ps2.register(newpoolname)
+    ps2.register(newpoolname, newpoolurl)
     assert op.exists(transpath(newpoolname, poolpath))
     assert len(ps2.getPools()) == 2
     assert ps2.getPools()[1] == newpoolname
@@ -451,7 +499,7 @@ def test_ProductStorage_init():
         pass  # assert False, 'exception expected'
 
 
-def check_ps_func_for_pool(thepoolname, thepoolurl):
+def check_ps_func_for_pool(thepoolname, thepoolurl, *args):
     ps = ProductStorage(poolurl=thepoolurl)
     p1 = ps.getPools()[0]
     # get the pool object
@@ -464,7 +512,7 @@ def check_ps_func_for_pool(thepoolname, thepoolurl):
     ref = ps.save(x)
     # ps has 1 prod
     assert ref.urn == 'urn:' + thepoolname + ':' + pcq + ':0'
-    checkdbcount(1, thepoolurl, pcq, 0)
+    checkdbcount(1, thepoolurl, pcq, 0, *args)
 
     # save more
     # one by one
@@ -476,8 +524,8 @@ def check_ps_func_for_pool(thepoolname, thepoolurl):
         x2.append(tmp)
         ref2.append(ps.save(tmp, tag='t' + str(d)))
 
-    checkdbcount(q, thepoolurl, pcq, q - 1)
-    checkdbcount(1, thepoolurl, fullname(MapContext), 0)
+    checkdbcount(q, thepoolurl, pcq, q - 1, *args)
+    checkdbcount(1, thepoolurl, fullname(MapContext), 0, *args)
     # save many in one go
     m, x3 = 2, []
     n = q + m
@@ -488,8 +536,8 @@ def check_ps_func_for_pool(thepoolname, thepoolurl):
     x2 += x3  # there are n prods in x2
     # check refs
     assert len(ref2) == n
-    checkdbcount(n, thepoolurl, pcq, n)
-    checkdbcount(1, thepoolurl, fullname(MapContext), 0)
+    checkdbcount(n, thepoolurl, pcq, n, *args)
+    checkdbcount(1, thepoolurl, fullname(MapContext), 0, *args)
 
     # tags
     ts = ps.getAllTags()
@@ -502,8 +550,8 @@ def check_ps_func_for_pool(thepoolname, thepoolurl):
     assert u[0] == ref2[q].urn
 
     # access resource
-    checkdbcount(n, thepoolurl, pcq, n)
-    checkdbcount(1, thepoolurl, fullname(MapContext), 0)
+    checkdbcount(n, thepoolurl, pcq, n, *args)
+    checkdbcount(1, thepoolurl, fullname(MapContext), 0, *args)
     # get ref from urn
     pref = ps.load(ref2[n - 2].urn)
     assert pref == ref2[n - 2]
@@ -519,13 +567,13 @@ def check_ps_func_for_pool(thepoolname, thepoolurl):
     # DB shows less in record
     # current serial number not changed
     # number of items decreased by 1
-    checkdbcount(n - 1, thepoolurl, pcq, n)
-    checkdbcount(1, thepoolurl, fullname(MapContext), 0)
+    checkdbcount(n - 1, thepoolurl, pcq, n, *args)
+    checkdbcount(1, thepoolurl, fullname(MapContext), 0, *args)
 
     # clean up a pool
     ps.wipePool()
-    checkdbcount(0, thepoolurl, pcq)
-    assert len(ps.getPool(thepoolname)._urns) == 0
+    checkdbcount(0, thepoolurl, pcq, -1, *args)
+    assert ps.getPool(thepoolname).isEmpty()
 
 
 def test_ProdStorage_func_local_mem():
@@ -534,7 +582,7 @@ def test_ProdStorage_func_local_mem():
     thepoolpath = '/tmp'
     thepoolurl = 'file://' + thepoolpath + '/' + thepoolname
     cleanup(thepoolurl, thepoolname)
-    check_ps_func_for_pool(thepoolname, thepoolurl)
+    check_ps_func_for_pool(thepoolname, thepoolurl, None)
 
     # mempool
     thepoolname = DEFAULT_MEM_POOL
@@ -542,26 +590,39 @@ def test_ProdStorage_func_local_mem():
     thepoolurl = 'mem://' + thepoolpath + thepoolname
 
     cleanup(thepoolurl, thepoolname)
-    check_ps_func_for_pool(thepoolname, thepoolurl)
+    check_ps_func_for_pool(thepoolname, thepoolurl, None)
 
 
-def test_ProdStorage_func_http():
+def test_ProdStorage_func_http(setup, userpass):
+
+    aburl, hdr = setup
     # httpclientpool
     thepoolname = 'testhttppool'
-    thepoolurl = pc['httphost'] + pc['baseurl'] + '/' + thepoolname
+    thepoolurl = aburl + '/' + thepoolname
 
     cleanup(thepoolurl, thepoolname)
-    check_ps_func_for_pool(thepoolname, thepoolurl)
+    # First test registering with local pstor will also register on server
+    pool = HttpClientPool(poolurl=thepoolurl)
+    # not exist
+    with pytest.raises(RuntimeError):
+        assert pool.isEmpty()
+    # register
+    pstore = ProductStorage(pool=pool)
+    assert pool.isEmpty()
+    pstore.unregister(thepoolname)
+    # not exist
+    with pytest.raises(RuntimeError):
+        assert pool.isEmpty()
+    check_ps_func_for_pool(thepoolname, thepoolurl, userpass)
 
 
-def test_ProdStorage_func_server():
+def test_ProdStorage_func_server(local_pools_dir):
     # httppool , the http server-side pool
     thepoolname = 'testserverpool'
-    thepoolurl = 'server://'+pc['server_poolpath'] + \
-        pc['baseurl'] + '/' + thepoolname
+    thepoolurl = 'server://' + local_pools_dir + '/' + thepoolname
 
     cleanup(thepoolurl, thepoolname)
-    check_ps_func_for_pool(thepoolname, thepoolurl)
+    check_ps_func_for_pool(thepoolname, thepoolurl, None)
 
 
 def test_LocalPool():
@@ -569,7 +630,7 @@ def test_LocalPool():
     thepoolpath = '/tmp'
     thepoolurl = 'file://' + thepoolpath + '/' + thepoolname
 
-    ps = ProductStorage(thepoolname)
+    ps = ProductStorage(thepoolname, thepoolurl)
     p1 = ps.getPools()[0]
     # get the pool object
     pspool = ps.getPool(p1)
@@ -784,17 +845,18 @@ def doquery(poolpath, newpoolpath):
     chk(res[0], rec1[3])
     chk(res[1], rec1[4])
 
-    # same as above but query is a function
-    def t(m):
-        import re
-        return re.match('.*n.1.*', m['instrument'].value)
+    if not newpoolpath.startswith('http'):
+        # same as above but query is a function
+        def t(m):
+            import re
+            return re.match('.*n.1.*', m['instrument'].value)
 
-    q = MetaQuery(Product, t)
-    res = pstore.select(q)
-    # [3,4]
-    assert len(res) == 2, str(res)
-    chk(res[0], rec1[3])
-    chk(res[1], rec1[4])
+        q = MetaQuery(Product, t)
+        res = pstore.select(q)
+        # [3,4]
+        assert len(res) == 2, str(res)
+        chk(res[0], rec1[3])
+        chk(res[1], rec1[4])
 
     # same as above but query is on the product. this is slow.
     q = AbstractQuery(Product, 'p', '"n 1" in p.instrument')
@@ -806,6 +868,7 @@ def doquery(poolpath, newpoolpath):
 
 
 def test_query_local_mem():
+    cleanup()
     thepoolpath = '/tmp'
     doquery('file://'+thepoolpath, 'file://'+thepoolpath)
     doquery('mem://'+thepoolpath, 'mem://'+thepoolpath)
@@ -813,13 +876,15 @@ def test_query_local_mem():
     doquery('mem://'+thepoolpath, 'file://'+thepoolpath)
 
 
-def test_query_http():
+def test_query_http(setup):
+
+    aburl, hdrs = setup
+    aburl = aburl.rstrip('/')
+    cleanup()
     lpath = '/tmp'
-    thepoolpath = pc['node']['host']+':' + \
-        str(pc['node']['port']) + pc['baseurl']
-    doquery('http://'+thepoolpath, 'http://'+thepoolpath)
-    doquery('file://'+lpath, 'http://'+thepoolpath)
-    doquery('mem://'+lpath, 'http://'+thepoolpath)
+    doquery(aburl, aburl)
+    doquery('file://'+lpath, aburl)
+    doquery('mem://'+lpath, aburl)
 
 
 def test_RefContainer():
@@ -915,13 +980,13 @@ def test_MapContext():
     # stored prod
     thepoolname = 'pool_' + getpass.getuser()
     thepoolpath = '/tmp'
-    thepoolurl = 'file://' + thepoolname + '/'+thepoolname
+    thepoolurl = 'file://' + thepoolpath + '/'+thepoolname
     # create a prooduct
     x = Product(description='save me in store')
     # remove existing pools in memory
     PoolManager().removeAll()
     # create a product store
-    pstore = ProductStorage(thepoolname)
+    pstore = ProductStorage(thepoolname, thepoolurl)
     assert len(pstore.getPools()) == 1
     assert pstore.getWritablePool() == thepoolname
     assert op.isdir(transpath(thepoolname, thepoolpath))
@@ -999,7 +1064,7 @@ def test_realistic():
     # remove existing pools in memory
     PoolManager.removeAll()
     # clean up possible garbage of previous runs. use class method to avoid reading pool hk info during ProdStorage initialization.
-    pstore = ProductStorage(pool=poolname)  # on disk
+    pstore = ProductStorage(poolurl=poolurl)  # on disk
     pstore.wipePool()
 
     p1 = Product(description='p1')
